/* Copyright (c) 2002 Bruce Spencer, National Research Council, Canada */
import java.util.*;
import java.io.*;

class Heap{
  private Vector vec;
  Heap(int initialSize){
    vec = new Vector(initialSize);
  }
  Heap(){
    vec = new Vector();
  }
  boolean isEmpty(){
    return vec.isEmpty();
  }
  void addAll(Comparable[] anArray){
    for(int i = 0; i < anArray.length; i++){
      vec.add(anArray[i]);
    }
    heapify();
  }
  void addAll(Vector comps){
    Iterator it = comps.iterator();
    while(it.hasNext()){
      vec.add(it.next());
    }
    heapify();
  }
  private void heapify(){
    int halfSize = (vec.size()+1)/2-1;
    for(int i = halfSize; i >= 0; i--){
      percDown(i);
    }
  }
  private void percUp(int i){
    //assumes 0<=i<vec.size
    if(i > 0){
      Comparable ati = (Comparable) vec.elementAt(i);
      int parent = (i+1)/2-1;
      boolean searching = true;
      while(searching){
	Comparable atp = (Comparable) vec.elementAt(parent);
	searching = ati.compareTo(atp)<0;
	if(searching){
	  vec.setElementAt(atp,i);
	  searching = parent>0;
	  i = parent;
	  parent = (parent+1)/2-1;
	}
      }
      vec.setElementAt(ati,i);
    }
  }
  public Comparable removeSmallest(){
    //assumes has been heapified
    Comparable smallest = (Comparable) vec.elementAt(0);
    if(vec.size()==1){
      vec.removeElementAt(0);
    }
    else {
      Comparable lastmost = (Comparable) vec.elementAt(vec.size() - 1);
      vec.removeElementAt(vec.size()-1);
      vec.setElementAt(lastmost, 0);
      percDown(0);
    }
    return smallest;
  }
  private void percDown(int i){
    int leftChild = 2*i+1;
    if(leftChild < vec.size()){
      Comparable ati = (Comparable) vec.elementAt(i);
      int rightChild = leftChild+1;
      if(rightChild == vec.size()){
	Comparable atLeft = (Comparable) vec.elementAt(leftChild);
	if(ati.compareTo(atLeft)>0){
	  vec.setElementAt(ati, leftChild);
	  vec.setElementAt(atLeft, i);
	}
      }
      else {
	Comparable atLeft = (Comparable) vec.elementAt(leftChild);
	Comparable atRight = (Comparable) vec.elementAt(rightChild);
	if(ati.compareTo(atLeft)>0){
	  if(ati.compareTo(atRight)>0){
	    if(atLeft.compareTo(atRight)>0){
	      vec.setElementAt(atRight, i);
	      vec.setElementAt(ati, rightChild);
	      percDown(rightChild);
	    }
	    else {
	      vec.setElementAt(atLeft, i);
	      vec.setElementAt(ati, leftChild);
	      percDown(leftChild);
	    }
	  }
	  else {
	    vec.setElementAt(atLeft, i);
	    vec.setElementAt(ati, leftChild);
	    percDown(leftChild);
	  }
	}
	else if(ati.compareTo(atRight)>0){
	  vec.setElementAt(atRight, i);
	  vec.setElementAt(ati, rightChild);
	  percDown(rightChild);
	}
      }
    }
  }
  public void add(Comparable c){
    vec.add(c);
    percUp(vec.size()-1);
  }
  public boolean check(){
    //destuctively checks for sanity
    Comparable prev;
    if(!isEmpty()){
      prev = removeSmallest();
      while(!isEmpty()){
	Comparable curr = (Comparable) removeSmallest();
	if(curr.compareTo(prev)<0){
	  return false;
	}
	System.out.println("extracted " + curr);
	prev = curr;
      }
    }
    return true;
  }

  private StringBuffer sb;
  public String toString(){
    sb = new StringBuffer();
    toString(0, 0);
    return new String(sb);
  }

  private void toString(int node, int indent){
    if(node < vec.size()){
      int leftChild = node*2+1;
      toString(leftChild, indent+2);
      indent(indent);
      sb.append(vec.get(node).toString());
      int rightChild = leftChild+1;
      toString(rightChild, indent+2);
    }
  }

  private void indent(int n){
    sb.append("\n");
    for(int i = 0; i < n; i++){
      sb.append("  ");
    }
  }
}
